<?php
class ivTheme
{
	/**
	 * Theme name
	 * @var string
	 */
	var $_name;
	
	/**
	 * Theme config filename
	 * @var string
	 */
	var $_themeConfigFilename = 'themeConfig.xml';
	
	/**
	 * Constructor
	 *
	 * @param string $name
	 */
	function ivTheme($name)
	{
		$this->__construct($name);
	}
	
	/**
	 * Constructor
	 *
	 * @param string $name
	 */
	function __construct($name)
	{
		$this->_name = $name;
	}
	
	/**
	 * Returns theme's name
	 *
	 * @return string
	 */
	function getName()
	{
		return $this->_name;
	}

	/**
	 * Checks if theme exists
	 *
	 * @return boolean
	 */
	function themeExists()
	{
		return in_array($this->_name, ivTheme::getAllThemes());
	}
	
	/**
	 * Returns theme's config
	 *
	 * @return ivXml|boolean
	 */
	function getConfig()
	{
		if (!$this->themeExists()) {
			return false;
		}
		$configFile = $this->_getPath() . $this->_themeConfigFilename;
		$descXml = ivXml::readFromFile($configFile, DEFAULT_THEME_CONFIG_FILE);
		$userConfigFile = $this->_getUserPath() . $this->_themeConfigFilename;
		$xml = &ivXml::readFromFile($userConfigFile, $descXml);
		return $xml;
	}
	
	/**
	 * Returns full config
	 *
	 * @return ivXml|boolean
	 */
	function getFullConfig()
	{
		if (!$this->themeExists()) {
			return false;
		}
		$path = $this->_getPath() . $this->_themeConfigFilename;
		$userPath = $this->_getUserPath() . $this->_themeConfigFilename;
		$descConfigXml = ivXml::readFromFile(DEFAULT_CONFIG_FILE);
		$descThemeXml = ivXml::readFromFile(DEFAULT_THEME_CONFIG_FILE);
		$descXml = $this->_mergeXml($descConfigXml, $descThemeXml);
		$descXmlConfig = ivXml::readFromFile(CONFIG_FILE, $descXml);
		$descXmlConfigTheme = ivXml::readFromFile($path, $descXmlConfig);
		$xml = &ivXml::readFromFile($userPath, $descXmlConfigTheme);
		return $xml;
	}
	
	/**
	 * Returns theme's CSS
	 *
	 * Reads theme's CSS and populates it with blocks from default.css
	 * 
	 * @return string
	 */
	function getStyle()
	{
		$defaultCss = file_get_contents(INCLUDE_DIR . 'default.css');
		preg_match_all('/^\s*([^\s]*)\s*\{.*?\}\s*$/ms', $defaultCss, $matches);
		$css = file_get_contents($this->_getPath() . 'imagevue.css');
		foreach ($matches[1] as $key => $match) {
			if (!preg_match('/(^|[^\w]+)' . $match . '([^\w]+|$)/', $css)) {
				$css .= "\n" . $matches[0][$key];
			}
		}
		return $css;
	}

	/**
	 * Saves theme's CSS
	 *
	 * @param  string  $css
	 * @return boolean
	 */
	function setStyle($css)
	{
		return @file_put_contents($this->_getPath() . 'imagevue.css', $css);
	}
	
	/**
	 * Returns upload directory path for current theme
	 *
	 * @return string
	 */
	function getUploadDirectory()
	{
		return $this->_getPath();
	}
	
	/**
	 * Returns list of all installed themes
	 *
	 * @static
	 * @return array
	 */
	function getAllThemes()
	{
		$list = array();
		if ($handle = opendir(THEMES_DIR)) {
			while (false !== ($file = readdir($handle))) { 
				if (substr($file, 0, 1) != '.' && is_dir(THEMES_DIR . $file)) { 
					$list[] = $file;
				}
			}
			closedir($handle); 
		}
		return $list;
	}
	
	/**
	 * Copies theme
	 *
	 * @param  string  $name New theme's name
	 * @return boolean
	 */
	function copyTo($name)
	{
		if (!$this->themeExists() || in_array($name, ivTheme::getAllThemes())) {
			return false;
		}

		$themeDir = $this->_getPath();
		$newThemeDir = $this->_getPath($name);
		$result = mkdirRecursive($newThemeDir);
		$handle = opendir($themeDir);
		while (false !== ($file = readdir($handle))) {
			if (!in_array($file, array('.', '..'))) {
				$fullPath = $themeDir . $file;
				if (is_file($fullPath)) {
					$result &= @copy($fullPath, $newThemeDir . $file);
				}
			}
		}
		closedir($handle);

		$userThemeDir = $this->_getUserPath();
		$newUserThemeDir = $this->_getUserPath($name);
		$result &= mkdirRecursive($newUserThemeDir);
		$handle = opendir($userThemeDir);
		while (false !== ($file = readdir($handle))) {
			if (!in_array($file, array('.', '..'))) {
				$fullPath = $userThemeDir . $file;
				if (is_file($fullPath)) {
					$result &= @copy($fullPath, $newUserThemeDir . $file);
				}
			}
		}
		closedir($handle);

		return $result;
	}
	
	/**
	 * Deletes current theme
	 *
	 * @return boolean
	 */
	function delete()
	{
		if (!$this->themeExists()) {
			return false;
		}
		$result = true;

		$handle = opendir($this->_getPath());
		while (false !== ($file = readdir($handle))) {
			if (!in_array($file, array('.', '..'))) {
				$fullPath = $this->_getPath() . $file;
				if (is_file($fullPath)) {
					$result &= @unlink($fullPath);
				}
			}
		}
		closedir($handle);
		$result &= @rmdir($this->_getPath());

		$handle = opendir($this->_getUserPath());
		while (false !== ($file = readdir($handle))) {
			if (!in_array($file, array('.', '..'))) {
				$fullPath = $this->_getPath() . $file;
				if (is_file($fullPath)) {
					$result &= @unlink($fullPath);
				}
			}
		}
		closedir($handle);
		$result &= @rmdir($this->_getUserPath());

		return $result;
	}

	/**
	 * Merges two XMLs
	 *
	 * @access private
	 * @param  ivXml $xml1
	 * @param  ivXml $xml2
	 * @return ivXml
	 */
	function &_mergeXml($xml1, $xml2)
	{
		foreach ($xml2->toFlatTree() as $nodeItem) {
			if (!$nodeItem['node']->hasChildren()) {
				$xml1->add($nodeItem['path'], $nodeItem['node']);
			}
		}
		return $xml1;
	}

	/**
	 * Returns theme's path
	 *
	 * @param  string $name
	 * @return string
	 */
	function _getPath($name = null)
	{
		$name = is_null($name) ? $this->getName() : $name;
		return THEMES_DIR . ivFilepath::normalizeDir($name);
	}
	
	/**
	 * Returns theme's user path
	 *
	 * @param  string $name
	 * @return string
	 */
	function _getUserPath($name = null)
	{
		$name = is_null($name) ? $this->getName() : $name;
		return USER_DIR . ivFilepath::normalizeDir($name);
	}

}
?>